import { createClient } from 'redis';
import { redis as url } from '../config.js';

const client = createClient({
  url: url,
  retry_strategy: function(options) {
    if (options.error && options.error.code === "ECONNREFUSED") {
        return new Error("The server refused the connection");
    }
    if (options.total_retry_time > 1000 * 60 * 60) {
        return new Error("Retry time exhausted");
    }
    if (options.attempt > 10) {
        return undefined;
    }
    return Math.min(options.attempt * 100, 3000);
  },
});

client.on('error', err => console.log('Redis Client Error', err));

await client.connect();

export default client